local GD, C, L = unpack(select(2, ...))
local MODULE_NAME = "GoldDelta"


---------------------------------------------
-- VARIABLES
---------------------------------------------
local oldAmount


---------------------------------------------
-- GOLD DELTAS
---------------------------------------------
local function trackMoneyChanged()
    if ( oldAmount == nil ) then
        return
    end

    local newAmount = GetMoney()
    local delta = newAmount - oldAmount
    oldAmount = newAmount

    GoldDeltaDB.expiresAt = time() + GD.RELOAD_WINDOW
    GoldDeltaDB.delta = GoldDeltaDB.delta + delta
end

local function showDeltaTooltip(self)
    local color = ( GoldDeltaDB.delta < 0 ) and RED_FONT_COLOR or GREEN_FONT_COLOR
    GameTooltip:SetOwner(self, "ANCHOR_LEFT")
    GameTooltip:SetText(GetMoneyString(math.abs(GoldDeltaDB.delta)), color.r, color.g, color.b)
    GameTooltip:Show()
end


---------------------------------------------
-- INITIALIZE
---------------------------------------------
local function createDeltaFrame(parent)
    local frame = CreateFrame("Frame", nil, parent)
    frame:SetFrameLevel(100)
    frame:SetAllPoints(parent)
    frame:SetScript("OnEnter", showDeltaTooltip)
    frame:SetScript("OnLeave", GameTooltip_Hide)
    parent.DeltaFrame = frame
end

GD.RegisterCallback(MODULE_NAME, "initialize", function()
    GD.RegisterEvent(MODULE_NAME, "PLAYER_ENTERING_WORLD", function()
        oldAmount = GetMoney()
    end)
    GD.RegisterEvent(MODULE_NAME, "PLAYER_LOGOUT", function()
        GoldDeltaDB.expiresAt = time() + GD.RELOAD_WINDOW
    end)

    -- Watch combat log for healthstone created
    GD.RegisterEvent(MODULE_NAME, "PLAYER_MONEY", trackMoneyChanged)

    -- Hook MoneyFrames
    local frames = {
        ContainerFrame1MoneyFrame,
        BankFrameMoneyFrame,
        MerchantMoneyFrame,
        PetStableMoneyFrame,
    }
    for _,frame in ipairs(frames) do
        createDeltaFrame(frame)
    end

    -- Hook for frames that are loaded on-demand
    GD:InitOnDemand("Blizzard_AuctionUI", function()
        createDeltaFrame(AuctionsDepositMoneyFrame)
        createDeltaFrame(AuctionFrameMoneyFrame)
    end)
    GD:InitOnDemand("Blizzard_TrainerUI", function()
        createDeltaFrame(ClassTrainerMoneyFrame)
    end)
end)
