local ADDON_NAME, ADDON = ...
local MODULE_NAME = "addon.lua"

ADDON[1] = {} -- GD, Addon
ADDON[2] = {} -- C, Config
ADDON[3] = {} -- L, Locale
local GD, C, L = unpack(ADDON)

--[===[@debug@
_G["GD"] = GD --Expose internals on the Plugin lib
--@end-debug@]==]]===]


---------------------------------------------
-- CONSTANTS
---------------------------------------------
GD.ADDON_NAME = ADDON_NAME
GD.VERSION = GetAddOnMetadata(ADDON_NAME, "Version")
GD.RELOAD_WINDOW = 60


---------------------------------------------
-- EVENTS & CALLBACKS
---------------------------------------------
GD.callbacks = LibStub("CallbackHandler-1.0"):New(GD)
GD.eventCallbacks = LibStub("CallbackHandler-1.0"):New(GD, "RegisterEvent", "UnregisterEvent", false)

local frame = CreateFrame("frame")
frame:SetScript("OnEvent", function(self, event, ...)
    GD.eventCallbacks:Fire(event, ...)
end)

function GD.eventCallbacks:OnUsed(target, event)
    frame:RegisterEvent(event)
end
function GD.eventCallbacks:OnUnused(target, event)
    frame:UnregisterEvent(event)
end

function GD:InitOnDemand(addon, init)
    if IsAddOnLoaded(addon) then
        init()
    else
        local registrationKey = addon .. "@" .. GetTime()
        GD.RegisterEvent(registrationKey, "ADDON_LOADED", function(event, addonName)
            if ( addon == addonName ) then
                GD.UnregisterEvent(registrationKey, "ADDON_LOADED")
                init()
            end
        end)
    end
end


---------------------------------------------
-- INITIALIZE
---------------------------------------------
--[===[@alpha@
LoadAddOn("Blizzard_DebugTools")
--@end-alpha@]=]]===]

GD.RegisterEvent(MODULE_NAME, "ADDON_LOADED", function(event, addonName)
    if ( addonName == GD.ADDON_NAME ) then
        GD.UnregisterEvent(MODULE_NAME, "ADDON_LOADED")

        -- Recreate if nil or expired
        if ( not GoldDeltaDB or GoldDeltaDB.expiresAt <= time() ) then
            GoldDeltaDB = {
                expired = time() + GD.RELOAD_WINDOW,
                delta = 0
            }
        end

        -- Initialize addon modules
        GD.callbacks:Fire("initialize")
    end
end)
